<?php

namespace Drupal\testsuite\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\testsuite\LoadResourceService;
use Drupal\testsuite\TestMenuService;

/**
 * Configure example settings for this site.
 */
class CustomTestsForm extends ConfigFormBase {

  /**
   * Config settings.
   *
   * @var string
   */
  const SETTINGS = 'testsuite.custom';

  /**
   * @var \Drupal\testsuite\LoadResourceService
   */
  protected $_loadResourceService;

  /**
   * @var \Drupal\testsuite\TestMenuService
   */
  protected $_testMenuService;

  /**
   * @var array The array of files to create Resource with.
   */
  protected $_loadedFiles = [];

  /**
   * @var bool If there are test.
   */
  protected $_testsFound = true;

  /**
   * @param \Drupal\testsuite\LoadResourceService $loadResourceService
   * @param \Drupal\testsuite\TestMenuService $testMenuService
   */
  public function __construct(LoadResourceService $loadResourceService, TestMenuService $testMenuService) {
    $this->_testMenuService = $testMenuService;
    $this->_loadResourceService = $loadResourceService;
    $files = $this->_loadResourceService->getResource('form', 'custom', 'all', 'all', 'all');
    $moduleHandler = \Drupal::service('module_handler');
    if ($moduleHandler->moduleExists('custom_tests')) {
      if(count($files) > 0) {
        foreach($files as $file) {
          $className = basename($file, '.php');
          $class = '\Drupal\custom_tests\Tests\\'.$className;
          require_once $file;
          if(class_exists($class)) {
            $obj = new $class;
            if (\Drupal::moduleHandler()->moduleExists($obj->getModuleName()) || $obj->getModuleName() == '') {
              $this->_loadedFiles[$className] = array(
                'file' => $file,
                'moduleName' => $obj->getModuleName(),
                'name' => $obj->getName(),
                'description' => $obj->getDescription(),
                'class' => '\Drupal\custom_tests\Tests\\'.$className,
              );
            }
          }
        }
      } else if ($moduleHandler->moduleExists('custom_tests')) {
        $this->_testsFound = false;
        $this->_loadedFiles['tests_not_found'] = array(
          'name' => 'No custom tests found.'
        );
      } else {
        $this->_testsFound = false;
        $this->_loadedFiles['tests_not_found'] = array(
          'name' => 'Please enable custom_tests module to create custom tests.'
        );
      }
    } else {
        $this->_testsFound = false;
        $this->_loadedFiles['custom_tests_not_found'] = array(
          'name' => 'Please copy the custom_tests module into the modules/custom folder, rename custom_tests.info.yml.txt to custom_tests.info.yml and enable the module to create and use custom tests.'
        );
      }
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('testsuite.load_resource.service'),
      $container->get('testsuite.test_menu.service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'testsuite_custom_tests_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTINGS,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $top_menu = $this->_testMenuService->buildTestsMenu();

    if($this->_testsFound) {
      $container['submit'] = [
        '#type' => 'submit',
        '#attributes' => array(
          'class' => ['button', 'button--primary', 'test-suite-top-build-button']
        ),
        '#value' => $this->t('Build Report')
      ];
    }

    foreach($this->_loadedFiles as $className => $values) {
      if($className == 'custom_tests_not_found') {
        $container[$className] = [
          '#type' => 'markup',
          '#markup' => '<p>'.$this->t($values['name']).'</p>',
        ];
      } else if($className == 'tests_not_found') {
        $container[$className] = [
          '#type' => 'markup',
          '#markup' => '<p>'.$this->t($values['name']).'</p>',
        ];
      } else {
        $container[$className] = [
          '#type' => 'checkbox',
          '#title' => $this->t($values['name']),
          '#attributes' => array(
            'id' => $className,
            'name' => $className,
          ),
          '#default_value' => TRUE,
        ];
      }
    }

    if($this->_testsFound) {
      $container['submit_bottom'] = [
        '#type' => 'submit',
        '#attributes' => array(
          'class' => ['button', 'button--primary']
        ),
        '#value' => $this->t('Build Report')
      ];
    }

    $form['container'] = [
      '#type' => 'html_tag',
      '#tag' => 'div',
      '#attributes' => [
        'class' => ['test-suite-container']
      ],
      'children' => [
        $top_menu,
        [
          '#type' => 'html_tag',
          '#tag' => 'div',
          '#attributes' => ['class' => ['test-suite-custom-row']],
          'children' => [
            [
              '#type' => 'html_tag',
              '#tag' => 'div',
              '#attributes' => ['class' => ['test-suite-content']],
              'child' => $container
            ]
          ]
        ]
      ]
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    foreach($this->_loadedFiles as $className => $values) {
      if($form_state->hasValue($className) && $form_state->getValue($className) == TRUE) {
        require_once $values['file'];
        if(class_exists($values['class'])) {
          $obj = new $values['class'];
          $request = $this->getRequest();
          $results = $obj->runTest($request);
          $request->getSession()->set($className, $results);
          $form_state->setRedirect('testsuite.custom_report');
        }
      }
    }
  }
}
